/* SiteRefWalker Class for GoLive SDK Extensions
 * 2004 OUT Media Design GmbH
 * Author : Oliver Zahorka
 * Version: 1.0.1
 * History:
 * 2004-09-27 - oz - initial release
 * 2004-09-27 - oz - fixed progress log bug for unprocessed documents, added parameters procFileType and logNotProcessed
 *
 */

/* override needsProcessing() and processDocument() to
 * implement specific behavior in your extensions
 */

function SiteRefWalker () {
	this.convertcount=0;
	this.progressLog = null;
	this.procName = new Array(); // empty Array of names for processing types, must be filled
	this.procDocView = null;     // can process file with openMarkup
	this.procFileType = "html"; // the file types of documents accepted for processing [html,alias,mail,image]
	this.confirmProcess = true;
	this.showProcessLog = true;
	this.showEncodingAlert = false;
	this.refWalkerSaveDoc = false;
	this.doWorkgroupDownload = true;
	this.doWorkgroupCheckOut = false;
	this.logNotProcessed = false;

	this.procMsg = new Array("item"," -- "+module.localize("processed"));
	this.confirmMsg = " ";
	this.confirmMsg += " ";
	this.confirmMsg += module.localize("documents to process");
	this.confirmMsg += ".\n\n";
	this.confirmMsg += module.localize("This operation will permanently modify the selected documents and can neither be aborted nor undone.");
	this.confirmMsg += "\n\n";
	this.confirmMsg += module.localize("All open files will be closed.");
	this.confirmMsg += " ";
	this.confirmMsg += module.localize("Continue?");
}


SiteRefWalker.prototype.needsProcessing = function (theDoc, procType)
{
	var procOk = 0;

	/* start of custom document processing code */
	
	procOk = 1;
	
	/* end of custom document processing code */

	return procOk;
}


SiteRefWalker.prototype.processDocument = function (theDoc,procType)
{
	var isProcessed=0;
		
	/* start of custom document processing code */

	//theDoc.reformat();

	/* set message to be shown by Process Log */
	this.procMsg[0] = "item"; // set the icon to be shown
	this.procMsg[1] = module.localize("processed")+": "+this.procName[procType]; // set the text to be shown

	isProcessed = 1;

	/* end of custom document processing code */

	return isProcessed;
}



SiteRefWalker.prototype.processCollection = function (theColl,procType) {
	app.startProgress(module.localize("Prepare")+" "+this.procName[procType],module.localize("checking documents"),true);
	this.convertcount=this.collectionWalker(theColl,procType,false);
	app.stopProgress();
	if (this.convertcount) {
		if (this.confirmProcess && !confirm(this.convertcount+this.confirmMsg)) return;
		if (this.refWalkerSaveDoc) {
			/* close all open documents except site */
			for(var i=documents.length-1, doClose=true; i>=0; i--) {
				for(var j=0; j<websites.length && doClose; j++) {
					if(documents[i] == websites[j].document) doClose = false;
				}
				if (doClose) documents[i].close();
				doClose = true;
			}
		}
		if (this.showProcessLog) {
			this.progressLog = app.createProgressLog();
			this.progressLog.begin(this.procName[0],module.localize("processing documents")+"...",true);
			this.progressLog.addMessage("info", this.procName[procType]+": "+this.convertcount+" "+module.localize("documents to process")+".");
		} else if(!this.procDocView) {
			app.startProgress(this.procName[procType],module.localize("processing documents")+"...",true);
		}
		this.convertcount=this.collectionWalker(theColl,procType,true);
		
		if (this.showProcessLog) {
			this.progressLog.addMessage("info", module.localize("Finished")+": "+this.convertcount+" "+(this.convertcount!=1?module.localize("documents were processed"):module.localize("document was processed"))+".");
			this.progressLog.end();
			this.progressLog = null;
		} else if(!this.procDocView) {
			app.stopProgress();
		}
	} else {
		alert(module.localize("No documents need to be processed"));
	}
}


SiteRefWalker.prototype.collectionWalker = function (aSiteRefIterator,procType,doit)
{
	var count = 0; error = false;
	var curSiteRef = aSiteRefIterator.first();
	while (curSiteRef) {

		// for test run, download file from AWWS if not already available
		if (!doit && this.doWorkgroupDownload && !curSiteRef.isCheckedOut && !curSiteRef.isAvailableLocal) curSiteRef.downloadFromServer();
		
		// for process run, check out file from AWWS if possible
		if (doit && this.doWorkgroupCheckOut && !curSiteRef.isCheckedOut) curSiteRef.checkOut();

		if (curSiteRef.type == "folder") {
			count+=this.collectionWalker(curSiteRef.getFiles(this.procFileType+"+folder"),procType,doit);
		}
		else if (curSiteRef.type == this.procFileType)  {

			var curFile = JSXFile(curSiteRef.longUrl);
			var curDoc = curFile.openMarkup();
			if (curDoc) {
				var needsProcessing = this.needsProcessing(curDoc, procType);
				if (needsProcessing > 0) {
					if (doit) {
						if ((this.doWorkgroupCheckOut && curSiteRef.isAvailableLocal) || (!this.doWorkgroupCheckOut && curSiteRef.local)) {

							if (this.procDocView!=null) {
							/* In case the doc must be opened in "layout" or another view */
								var saveEncAlert = app.encodingAlert;
								app.encodingAlert = this.showEncodingAlert;
	
								curDoc.close();
								
								curDoc = curFile.openDocument();
								curDoc.view = this.procDocView;
	
								app.encodingAlert = saveEncAlert;
							}
							
							if (!this.procDocView && !this.showProcessLog) {
								// show progress bar if document is processed without opening and Process Log is off
								if (!app.setProgress(0,relativeURL(curSiteRef.longUrl,curSiteRef.siteDoc.site.longUrl))) return count; //check if user clicked stop
							}
							
							
							/**** Process the document ****/
							if (this.processDocument(curDoc,procType)) {
								count++;
								// only save if SiteRefWalker is set to do so
								if (this.refWalkerSaveDoc) curDoc.save();
							}
							
							// set message for Process Log
							this.procMsg[1] = curDoc.file.name+" -- "+this.procMsg[1];
						} else {
							// set error message for Process Log
							this.procMsg[0] = "error";
							this.procMsg[1] = module.localize("Workgroup document not available locally")+": " + curSiteRef.longUrl;
						}
					} else {
						count++;
						// set message for progress dialog
						if (!app.setProgress(0,relativeURL(curSiteRef.longUrl,curSiteRef.siteDoc.site.longUrl)))
							return count; // user clicked cancel button
					}
				} else if (needsProcessing < 0) {
					// set error message for Process Log
					this.procMsg[1] = curDoc.file.name+" -- "+this.procMsg[1];
				} else {
					// set not processed message for Process Log
					this.procMsg[1] = curDoc.file.name+" -- "+module.localize("no processing needed");
				}
				curDoc.close();
				// add message to the Process Log
				if ((needsProcessing || this.logNotProcessed) && this.showProcessLog && this.progressLog!=null) this.progressLog.addMessage(this.procMsg[0], this.procMsg[1], curFile.longUrl);
			} else {
				error = true;
			}
		} else {
			error = true;
		}
		if (error && this.showProcessLog && this.progressLog!=null) this.progressLog.addMessage("error",module.localize("Error opening document")+": " + curFile.longUrl);
		error = false;
		curSiteRef = aSiteRefIterator.next();
	}
	return count;
}



